require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
	class Game
		
		attr_reader :title

		def initialize(title)
			@title = title
			@players = []
		end

		def load_players(filename)
			File.readlines(filename).each do |line|
				#puts line
				
				add_player(Player.from_csv(line))
			end
		end

		def save_high_scores(filename="high_scores.txt")
			File.open(filename, "w") do |file|
				file.puts "#{@title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end

		def high_score_entry(player)
			format_string = "#{player.name}".ljust(25,".") + "#{player.health}"
			format_string
		end
		
		def add_player(player)
			@players << player
		end


		def play(rounds)

			#puts "There are #{@players.size} players in #{@title}:"
	#		@players.each do |name|
	#			puts name
	#		end

			1.upto(rounds) do |round|
				if block_given?
					if yield
						break
					end
				end
					
				puts "\nRound No. #{round}"
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end

			treasures = TreasureTrove::TREASURES
			puts "\nThere are #{treasures.size} treasures to be found"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points} points"
			end
				#if number_rolled >= 5	
	#				player.w00t
	#			elsif number_rolled <3 
	#				player.blam
	#			else
	#				puts "#{player.name} is skipped"
	#			end	
				# 
		end

		def print_stats
			puts "\nKnuckleheads Statistics:"
			strong_players, wimpy_players = @players.partition {|player| player.strong?} 
			
			puts "\n#{strong_players.length} strong players:" 
			strong_players.each do |player|
				print_name_and_health(player)
			end

			puts "\n#{wimpy_players.length} wimpy players:" 
			wimpy_players.each do |player|
				print_name_and_health(player)
			end

			puts "\nKnuckleheads High Scores:"
			@players.sort.each do |player|
				puts high_score_entry(player)
			end

			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.name} = #{treasure.points}"
				end
				puts "#{player.points} grand total points"
			end

			puts "Total treasure points found: #{total_points}"

		end

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})" 
		end

		def total_points	
			@players.reduce(0) {|sum,player| sum + player.points}
		end
	end
end

if __FILE__ == $0
	test_game = Game.new("Testing the Game")
	puts test_game.title
end
